@@ -1,7 +1,7 @@
 ///
 /// @file  LoadBalancer.hpp
 ///
-/// Copyright (C) 2017 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2019 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -33,25 +33,23 @@ struct Runtime
 class LoadBalancer
 {
 public:
-  LoadBalancer(maxint_t x, int64_t y, int64_t z, maxint_t s2_approx);
-  bool get_work(int64_t* low, int64_t* segments, int64_t* segment_size, maxint_t S2, Runtime& runtime);
-  maxint_t get_result() const;
+  LoadBalancer(maxint_t x, int64_t sieve_limit, maxint_t sum_approx);
+  bool get_work(int64_t* low, int64_t* segments, int64_t* segment_size, maxint_t sum, Runtime& runtime);
+  maxint_t get_sum() const;
 
 private:
-  void init_size();
   void update(int64_t* low, int64_t* segments, Runtime& runtime);
-  double get_next(Runtime& runtime) const;
+  void update_segments(Runtime& runtime);
   double remaining_secs() const;
 
   int64_t low_;
   int64_t max_low_;
-  int64_t z_;
+  int64_t sieve_limit_;
   int64_t segments_;
   int64_t segment_size_;
   int64_t max_size_;
-  int64_t smallest_hard_leaf_;
-  maxint_t s2_total_;
-  maxint_t s2_approx_;
+  maxint_t sum_;
+  maxint_t sum_approx_;
   double time_;
   S2Status status_;
 };
