@@ -3,7 +3,7 @@
 /// @brief The MpiLoadBalancer evenly distributes the computation
 ///        of the hard special leaves onto cluster nodes.
 ///
-/// Copyright (C) 2017 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2019 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -24,22 +24,21 @@ namespace primecount {
 class MpiLoadBalancer
 {
 public:
-  MpiLoadBalancer(maxint_t x, int64_t y, int64_t z, maxint_t s2_approx);
+  MpiLoadBalancer(maxint_t x, int64_t sieve_limit, maxint_t sum_approx);
   void get_work(MpiMsg* msg);
 
 private:
-  double get_next(Runtime& runtime) const;
+  void update_segments(Runtime& runtime);
   double remaining_secs() const;
 
-  int64_t z_;
-  int64_t limit_;
   int64_t low_;
   int64_t max_low_;
+  int64_t sieve_limit_;
   int64_t segments_;
   int64_t segment_size_;
-  int64_t smallest_hard_leaf_;
-  maxint_t s2_hard_;
-  maxint_t s2_approx_;
+  int64_t max_size_;
+  maxint_t sum_;
+  maxint_t sum_approx_;
   double time_;
   S2Status status_;
 };
