@@ -2,8 +2,8 @@
 /// @file  LoadBalancer.cpp
 /// @brief The LoadBalancer assigns work to the individual threads
 ///        in the computation of the special leaves in the
-///        Lagarias-Miller-Odlyzko and Deleglise-Rivat prime
-///        counting algorithms.
+///        Lagarias-Miller-Odlyzko, Deleglise-Rivat and Gourdon
+///        prime counting algorithms.
 ///
 ///        Simply parallelizing the computation of the special
 ///        leaves in the Lagarias-Miller-Odlyzko algorithm by
@@ -21,7 +21,7 @@
 ///        order to prevent that 1 thread will run much longer
 ///        than all the other threads.
 ///
-/// Copyright (C) 2018 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2019 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -44,34 +44,25 @@ using namespace std;
 namespace primecount {
 
 LoadBalancer::LoadBalancer(maxint_t x,
-                           int64_t y,
-                           int64_t z,
-                           maxint_t s2_approx) :
+                           int64_t sieve_limit,
+                           maxint_t sum_approx) :
   low_(0),
   max_low_(0),
-  z_(z),
+  sieve_limit_(sieve_limit),
   segments_(1),
-  s2_total_(0),
-  s2_approx_(s2_approx),
+  sum_(0),
+  sum_approx_(sum_approx),
   time_(get_time()),
   status_(x)
-{
-  init_size();
-  maxint_t x16 = iroot<6>(x);
-  double alpha = get_alpha(x, y);
-  smallest_hard_leaf_ = (int64_t) (x / (y * sqrt(alpha) * x16));
-}
-
-void LoadBalancer::init_size()
 {
   // start with a tiny segment_size as most
   // special leaves are in the first few segments
   // and we need to ensure that all threads are
   // assigned an equal amount of work
-  int64_t sqrtz = isqrt(z_);
-  int64_t log = ilog(sqrtz);
+  int64_t sqrt_limit = isqrt(sieve_limit);
+  int64_t log = ilog(sqrt_limit);
   log = max(log, 1);
-  segment_size_ = sqrtz / log;
+  segment_size_ = sqrt_limit / log;
 
   int64_t min_size = 1 << 9;
   segment_size_ = max(segment_size_, min_size);
@@ -81,38 +72,37 @@ void LoadBalancer::init_size()
   // into the CPUs L1 data cache
   int64_t l1_dcache_size = 1 << 15;
   max_size_ = l1_dcache_size * 30;
-  max_size_ = max(max_size_, sqrtz);
+  max_size_ = max(max_size_, sqrt_limit);
   max_size_ = Sieve::get_segment_size(max_size_);
 }
 
-maxint_t LoadBalancer::get_result() const
+maxint_t LoadBalancer::get_sum() const
 {
-  return s2_total_;
+  return sum_;
 }
 
 bool LoadBalancer::get_work(int64_t* low,
                             int64_t* segments,
                             int64_t* segment_size,
-                            maxint_t s2,
+                            maxint_t sum,
                             Runtime& runtime)
 {
   #pragma omp critical (get_work)
   {
-    s2_total_ += s2;
+    sum_ += sum;
 
     update(low, segments, runtime);
 
     *low = low_;
     *segments = segments_;
     *segment_size = segment_size_;
-
     low_ += segments_ * segment_size_;
 
     if (is_print())
-      status_.print(s2_total_, s2_approx_);
+      status_.print(sum_, sum_approx_);
   }
 
-  return *low <= z_;
+  return *low <= sieve_limit_;
 }
 
 void LoadBalancer::update(int64_t* low,
@@ -124,56 +114,65 @@ void LoadBalancer::update(int64_t* low,
     max_low_ = *low;
     segments_ = *segments;
 
+    // We only start increasing the segment_size and segments
+    // per thread once the first special leaves have been
+    // found. Near the start there is a very large number of
+    // leaves and we don't want a single thread to compute
+    // them all by himself (which would cause scaling issues).
+    if (sum_ == 0)
+      return;
+
     if (segment_size_ < max_size_)
       segment_size_ = min(segment_size_ * 2, max_size_);
     else
-    {
-      double next = get_next(runtime);
-      next = in_between(0.25, next, 2.0);
-      next *= segments_;
-      next = max(1.0, next);
-      segments_ = (int64_t) next;
-    }
+      update_segments(runtime);
   }
-
-  auto high = low_ + segments_ * segment_size_;
-
-  // Most hard special leaves are located just past
-  // smallest_hard_leaf_. In order to prevent assigning
-  // the bulk of work to a single thread we reduce
-  // the number of segments to a minimum.
-  //
-  if (smallest_hard_leaf_ >= low_ &&
-      smallest_hard_leaf_ <= high)
-  {
-    segments_ = 1;
-  }
-}
-
-double LoadBalancer::get_next(Runtime& runtime) const
-{
-  double min_secs = runtime.init * 10;
-  double run_secs = runtime.secs;
-
-  min_secs = max(min_secs, 0.01);
-  run_secs = max(run_secs, min_secs / 10);
-
-  double rem = remaining_secs();
-  double threshold = rem / 4;
-  threshold = max(threshold, min_secs);
-
-  return threshold / run_secs;
 }
 
 /// Remaining seconds till finished
 double LoadBalancer::remaining_secs() const
 {
-  double percent = status_.getPercent(low_, z_, s2_total_, s2_approx_);
-  percent = in_between(20, percent, 100);
-
+  double percent = status_.getPercent(low_, sieve_limit_, sum_, sum_approx_);
+  percent = in_between(10, percent, 100);
   double total_secs = get_time() - time_;
   double secs = total_secs * (100 / percent) - total_secs;
   return secs;
 }
 
+/// Increase or decrease the number of segments based on the
+/// remaining runtime. Near the end it is important that
+/// threads run only for a short amount of time in order to
+/// ensure all threads finish nearly at the same time.
+///
+void LoadBalancer::update_segments(Runtime& runtime)
+{
+  double rem = remaining_secs();
+  double threshold = rem / 4;
+  double min_secs = 0.01;
+
+  // Each thread should run at least 10x
+  // longer than its initialization time
+  threshold = max(threshold, runtime.init * 10);
+  threshold = max(threshold, min_secs);
+
+  // divider must not be 0
+  double divider = max(runtime.secs, min_secs / 10);
+  double factor = threshold / divider;
+
+  // Reduce the thread runtime if it is much
+  // larger than its initialization time
+  if (runtime.secs > min_secs &&
+      runtime.secs > runtime.init * 1000)
+  {
+    double old = factor;
+    factor = (runtime.init * 1000) / runtime.secs;
+    factor = min(factor, old);
+  }
+
+  factor = in_between(0.5, factor, 2.0);
+  double new_segments = round(segments_ * factor);
+  segments_ = (int64_t) new_segments;
+  segments_ = max(segments_, 1);
+}
+
 } // namespace
