@@ -1,10 +1,13 @@
 ///
 /// @file  S1.cpp
-/// @brief Calculate the contribution of the ordinary leaves
-///        in the Lagarias-Miller-Odlyzko and Deleglise-
-///        Rivat prime counting algorithms.
+/// @brief Calculate the contribution of the ordinary leaves in the
+///        Lagarias-Miller-Odlyzko and Deleglise-Rivat prime counting
+///        algorithms. Note that in the original paper from
+///        Lagarias-Miller-Odlyzko the ordinary leaves were named S1
+///        however in many newer papers the ordinary leaves are named
+///        S0. In primecount the ordinary leaves are named S1.
 ///
-/// Copyright (C) 2018 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2019 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -30,20 +33,20 @@ namespace {
 /// to the first b primes and calculate the sum of the
 /// ordinary leaves. This algorithm is described in section
 /// 2.2 of the paper: Douglas Staple, "The Combinatorial
-/// Algorithm For Computing pi(x)", arXiv:1503.01839, 6
-/// March 2015.
+/// Algorithm For Computing pi(x)", arXiv:1503.01839, 6 March
+/// 2015.
 ///
 template <int MU, typename T, typename P>
 T S1_thread(T x,
             int64_t y,
-            int64_t b,
+            uint64_t b,
             int64_t c,
             T square_free,
             vector<P>& primes)
 {
   T s1 = 0;
 
-  for (b += 1; b < (int64_t) primes.size(); b++)
+  for (b++; b < primes.size(); b++)
   {
     T next = square_free * primes[b];
     if (next > y) break;
@@ -64,15 +67,14 @@ X S1_OpenMP(X x,
             int64_t c,
             int threads)
 {
+  threads = ideal_num_threads(threads, y);
+
   auto primes = generate_primes<Y>(y);
-  int64_t pi_y = primes.size();
+  int64_t pi_y = primes.size() - 1;
   X s1 = phi_tiny(x, c);
 
-  int64_t thread_threshold = ipow(10, 6);
-  threads = ideal_num_threads(threads, y, thread_threshold);
-
   #pragma omp parallel for schedule(static, 1) num_threads(threads) reduction (+: s1)
-  for (int64_t b = c + 1; b < pi_y; b++)
+  for (int64_t b = c + 1; b <= pi_y; b++)
   {
     s1 -= phi_tiny(x / primes[b], c);
     s1 += S1_thread<1>(x, y, b, c, (X) primes[b], primes);
