@@ -3,7 +3,7 @@
 /// @brief  Parse command-line options for the primecount console
 ///         (terminal) application.
 ///
-/// Copyright (C) 2018 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2019 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -35,42 +35,49 @@ map<string, OptionID> optionMap =
 {
   { "-a", OPTION_ALPHA },
   { "--alpha", OPTION_ALPHA },
+  { "--alpha_y", OPTION_ALPHA_Y },
+  { "--alpha_z", OPTION_ALPHA_Z },
   { "-d", OPTION_DELEGLISE_RIVAT },
   { "--deleglise_rivat", OPTION_DELEGLISE_RIVAT },
-  { "--deleglise_rivat1", OPTION_DELEGLISE_RIVAT1 },
-  { "--deleglise_rivat2", OPTION_DELEGLISE_RIVAT2 },
-  { "--deleglise_rivat_parallel1", OPTION_DELEGLISE_RIVAT_PARALLEL1 },
-  { "--deleglise_rivat_parallel2", OPTION_DELEGLISE_RIVAT_PARALLEL2 },
+  { "--deleglise_rivat_64", OPTION_DELEGLISE_RIVAT_64 },
+  { "--deleglise_rivat_128", OPTION_DELEGLISE_RIVAT_128 },
+  { "-g", OPTION_GOURDON },
+  { "--gourdon", OPTION_GOURDON },
+  { "--gourdon_64", OPTION_GOURDON_64 },
+  { "--gourdon_128", OPTION_GOURDON_128 },
   { "-h", OPTION_HELP },
   { "--help", OPTION_HELP },
+  { "-l", OPTION_LEGENDRE },
   { "--legendre", OPTION_LEGENDRE },
   { "--lehmer", OPTION_LEHMER },
-  { "-l", OPTION_LMO },
   { "--lmo", OPTION_LMO },
   { "--lmo1", OPTION_LMO1 },
   { "--lmo2", OPTION_LMO2 },
   { "--lmo3", OPTION_LMO3 },
   { "--lmo4", OPTION_LMO4 },
   { "--lmo5", OPTION_LMO5 },
-  { "--lmo_parallel", OPTION_LMO_PARALLEL },
-  { "--Li", OPTION_LI },
-  { "--Li_inverse", OPTION_LIINV },
   { "-m", OPTION_MEISSEL },
   { "--meissel", OPTION_MEISSEL },
   { "-n", OPTION_NTHPRIME },
   { "--nthprime", OPTION_NTHPRIME },
   { "--number", OPTION_NUMBER },
-  { "--P2", OPTION_P2 },
-  { "--phi", OPTION_PHI },
-  { "--pi", OPTION_PI },
   { "-p", OPTION_PRIMESIEVE },
   { "--primesieve", OPTION_PRIMESIEVE },
+  { "--Li", OPTION_LI },
+  { "--Li_inverse", OPTION_LIINV },
   { "--Ri", OPTION_RI },
   { "--Ri_inverse", OPTION_RIINV },
+  { "--phi", OPTION_PHI },
+  { "--P2", OPTION_P2 },
   { "--S1", OPTION_S1 },
   { "--S2_easy", OPTION_S2_EASY },
   { "--S2_hard", OPTION_S2_HARD },
   { "--S2_trivial", OPTION_S2_TRIVIAL },
+  { "--AC", OPTION_AC },
+  { "--B", OPTION_B },
+  { "--D", OPTION_D },
+  { "--Phi0", OPTION_PHI0 },
+  { "--Sigma", OPTION_SIGMA },
   { "-s", OPTION_STATUS },
   { "--status", OPTION_STATUS },
   { "--test", OPTION_TEST },
@@ -109,7 +116,7 @@ void optionStatus(Option& opt,
 }
 
 /// e.g. "--thread=4" -> return "--thread"
-string getOption(string str)
+string getOption(const string& str)
 {
   size_t pos = str.find_first_of("=0123456789");
 
@@ -120,7 +127,7 @@ string getOption(string str)
 }
 
 /// e.g. "--thread=4" -> return "4"
-string getValue(string str)
+string getValue(const string& str)
 {
   size_t pos = str.find_first_of("0123456789");
 
@@ -168,6 +175,8 @@ CmdOptions parseOptions(int argc, char* argv[])
     switch (optionMap[opt.opt])
     {
       case OPTION_ALPHA:   set_alpha(stod(opt.val)); break;
+      case OPTION_ALPHA_Y: set_alpha_y(stod(opt.val)); break;
+      case OPTION_ALPHA_Z: set_alpha_z(stod(opt.val)); break;
       case OPTION_NUMBER:  numbers.push_back(opt.to<maxint_t>()); break;
       case OPTION_THREADS: set_num_threads(opt.to<int>()); break;
       case OPTION_PHI:     opts.a = opt.to<int64_t>(); opts.option = OPTION_PHI; break;
