@@ -26,9 +26,10 @@ const string helpMenu
   "Options:\n"
   "\n"
   "  -d,    --deleglise_rivat  Count primes using Deleglise-Rivat algorithm\n"
-  "         --legendre         Count primes using Legendre's formula\n"
+  "  -g,    --gourdon          Count primes using Xavier Gourdon's algorithm\n"
+  "  -l,    --legendre         Count primes using Legendre's formula\n"
   "         --lehmer           Count primes using Lehmer's formula\n"
-  "  -l,    --lmo              Count primes using Lagarias-Miller-Odlyzko\n"
+  "         --lmo              Count primes using Lagarias-Miller-Odlyzko\n"
   "  -m,    --meissel          Count primes using Meissel's formula\n"
   "         --Li               Approximate pi(x) using the logarithmic integral\n"
   "         --Li_inverse       Approximate the nth prime using Li^-1(x)\n"
@@ -46,19 +47,26 @@ const string helpMenu
   "  -v,    --version          Print version and license information\n"
   "  -h,    --help             Print this help menu\n"
   "\n"
-  "Advanced Deleglise-Rivat options:\n"
+  "Advanced options for the Deleglise-Rivat algorithm:\n"
   "\n"
   "  -a<N>, --alpha=<N>        Tuning factor, 1 <= alpha <= x^(1/6)\n"
-  "         --P2               Only compute the 2nd partial sieve function\n"
-  "         --S1               Only compute the ordinary leaves\n"
-  "         --S2_trivial       Only compute the trivial special leaves\n"
-  "         --S2_easy          Only compute the easy special leaves\n"
-  "         --S2_hard          Only compute the hard special leaves\n"
+  "         --P2               Compute the 2nd partial sieve function\n"
+  "         --S1               Compute the ordinary leaves\n"
+  "         --S2_trivial       Compute the trivial special leaves\n"
+  "         --S2_easy          Compute the easy special leaves\n"
+  "         --S2_hard          Compute the hard special leaves\n"
   "\n"
-  "Examples:\n"
+  "Advanced options for Xavier Gourdon's algorithm:\n"
   "\n"
-  "  primecount 1e13\n"
-  "  primecount 1e13 --nthprime --threads=4"
+  "         --alpha_y=<N>      Tuning factor, 1 <= alpha_y <= x^(1/6)\n"
+  "                            with y = x^(1/3) * alpha_y\n"
+  "         --alpha_z=<N>      Tuning factor, 1 <= alpha_z <= x^(1/6)\n"
+  "                            with z = y * alpha_z\n"
+  "         --AC               Compute Gourdon's A + C formulas\n"
+  "         --B                Compute Gourdon's B formula\n"
+  "         --D                Compute Gourdon's D formula\n"
+  "         --Phi0             Compute the Phi0 formula\n"
+  "         --Sigma            Compute the 7 Sigma formulas\n"
 );
 
 const string versionInfo
