@@ -13,7 +13,12 @@
 ///        POPCNT instruction. Hence this implementation does not use
 ///        a binary indexed tree.
 ///
-/// Copyright (C) 2018 Kim Walisch, <kim.walisch@gmail.com>
+///        This implementation is based on the paper:
+///        Tomás Oliveira e Silva, Computing pi(x): the combinatorial
+///        method, Revista do DETUA, vol. 4, no. 6, March 2006,
+///        pp. 759-768.
+///
+/// Copyright (C) 2019 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -41,9 +46,18 @@ using namespace primecount;
 
 namespace {
 
-/// Compute the contribution of the hard special leaves
-/// using a sieve. Each thread processes the interval
-/// [low, low + segments * segment_size[
+/// Compute the contribution of the hard special leaves using a
+/// segmented sieve. Each thread processes the interval
+/// [low, low + segments * segment_size[.
+///
+/// Note that in the Deleglise-Rivat paper it is suggested to use a
+/// segment size of y. In practice however this uses too much memory
+/// especially when using multi-threading. Hence we are using a
+/// segment size of sqrt(z) as suggested in Xavier Gourdon's paper.
+/// In primecount's implementation a segment size of sqrt(z) seems
+/// ideal since slightly increasing the segment size decreases
+/// performance because of cache misses and slightly decreasing the
+/// segment size also decreases performance.
 ///
 template <typename T, typename FactorTable, typename Primes>
 T S2_hard_thread(T x,
@@ -58,14 +72,16 @@ T S2_hard_thread(T x,
                  Primes& primes,
                  Runtime& runtime)
 {
+  T s2_hard = 0;
+  int64_t pi_sqrty = pi[isqrt(y)];
   int64_t low1 = max(low, 1);
   int64_t limit = min(low + segments * segment_size, z + 1);
-  int64_t max_b = pi[min(isqrt(x / low1), isqrt(z), y)];
-  int64_t pi_sqrty = pi[isqrt(y)];
-  T s2_hard = 0;
+  int64_t max_b = pi[min3(isqrt(x / low1), isqrt(z), y)];
+  int64_t min_b = pi[min(z / limit, primes[max_b])];
+  min_b = max(c, min_b) + 1;
 
-  if (c > max_b)
-    return s2_hard;
+  if (min_b > max_b)
+    return 0;
 
   runtime.init_start();
   Sieve sieve(low, segment_size, max_b);
@@ -79,22 +95,21 @@ T S2_hard_thread(T x,
     int64_t high = min(low + segment_size, limit);
     low1 = max(low, 1);
 
-    // pre-sieve multiples of first c primes
-    sieve.pre_sieve(c, low, high);
-
+    sieve.pre_sieve(primes, min_b - 1, low, high);
     int64_t count_low_high = sieve.count((high - 1) - low);
-    int64_t b = c + 1;
+    int64_t b = min_b;
 
     // For c + 1 <= b <= pi_sqrty
-    // Find all special leaves: n = primes[b] * m
-    // which satisfy: mu[m] != 0 && primes[b] < lpf[m] && low <= (x / n) < high
+    // Find all special leaves in the current segment that are
+    // composed of a prime and a square free number:
+    // low <= x / (primes[b] * m) < high
     for (int64_t end = min(pi_sqrty, max_b); b <= end; b++)
     {
       int64_t prime = primes[b];
-      T x2 = x / prime;
-      int64_t x2_div_high = min(fast_div(x2, high), y);
-      int64_t min_m = max(x2_div_high, y / prime);
-      int64_t max_m = min(fast_div(x2, low1), y);
+      T xp = x / prime;
+      int64_t xp_div_high = min(fast_div(xp, high), y);
+      int64_t min_m = max(xp_div_high, y / prime);
+      int64_t max_m = min(fast_div(xp, low1), y);
       int64_t count = 0;
       int64_t start = 0;
 
@@ -106,34 +121,36 @@ T S2_hard_thread(T x,
 
       for (int64_t m = max_m; m > min_m; m--)
       {
-        if (prime < factor.lpf(m))
+        // mu(m) != 0 && prime < lpf(m)
+        if (prime < factor.mu_lpf(m))
         {
           int64_t fm = factor.get_number(m);
-          int64_t xn = (int64_t) fast_div(x2, fm);
-          int64_t stop = xn - low;
+          int64_t xpm = fast_div64(xp, fm);
+          int64_t stop = xpm - low;
           count += sieve.count(start, stop, low, high, count, count_low_high);
           start = stop + 1;
-          int64_t phi_xn = phi[b] + count;
+          int64_t phi_xpm = phi[b] + count;
           int64_t mu_m = factor.mu(m);
-          s2_hard -= mu_m * phi_xn;
+          s2_hard -= mu_m * phi_xpm;
         }
       }
 
       phi[b] += count_low_high;
-      count_low_high -= sieve.cross_off(b, prime);
+      count_low_high -= sieve.cross_off_count(prime, b);
     }
 
     // For pi_sqrty < b <= pi_sqrtz
-    // Find all hard special leaves: n = primes[b] * primes[l]
-    // which satisfy: low <= (x / n) < high
+    // Find all special leaves in the current segment
+    // that are composed of 2 primes:
+    // low <= x / (primes[b] * primes[l]) < high
     for (; b <= max_b; b++)
     {
       int64_t prime = primes[b];
-      T x2 = x / prime;
-      int64_t x2_div_low = min(fast_div(x2, low1), y);
-      int64_t x2_div_high = min(fast_div(x2, high), y);
-      int64_t l = pi[min(x2_div_low, z / prime)];
-      int64_t min_hard = max(x2_div_high, prime);
+      T xp = x / prime;
+      int64_t xp_div_low = min(fast_div(xp, low1), y);
+      int64_t xp_div_high = min(fast_div(xp, high), y);
+      int64_t l = pi[min(xp_div_low, z / prime)];
+      int64_t min_hard = max(xp_div_high, prime);
       int64_t count = 0;
       int64_t start = 0;
 
@@ -142,16 +159,16 @@ T S2_hard_thread(T x,
 
       for (; primes[l] > min_hard; l--)
       {
-        int64_t xn = (int64_t) fast_div(x2, primes[l]);
-        int64_t stop = xn - low;
+        int64_t xpq = fast_div64(xp, primes[l]);
+        int64_t stop = xpq - low;
         count += sieve.count(start, stop, low, high, count, count_low_high);
         start = stop + 1;
-        int64_t phi_xn = phi[b] + count;
-        s2_hard += phi_xn;
+        int64_t phi_xpq = phi[b] + count;
+        s2_hard += phi_xpq;
       }
 
       phi[b] += count_low_high;
-      count_low_high -= sieve.cross_off(b, prime);
+      count_low_high -= sieve.cross_off_count(prime, b);
     }
 
     next_segment:;
@@ -161,11 +178,22 @@ T S2_hard_thread(T x,
 }
 
 /// Calculate the contribution of the hard special leaves.
-/// This is a parallel implementation with advanced load balancing.
-/// As most special leaves tend to be in the first segments we
-/// start off with a small segment size and few segments
-/// per thread, after each iteration we dynamically increase
-/// the segment size and the number of segments.
+///
+/// This is a parallel S2_hard(x, y) implementation with advanced load
+/// balancing. As most special leaves tend to be in the first segments
+/// we start off with a tiny segment size and one segment per thread.
+/// After each iteration we dynamically increase the segment size (until
+/// it reaches some limit) or the number of segments.
+///
+/// S2_hard(x, y) has been parallelized using an idea devised by Xavier
+/// Gourdon. The idea is to make the individual threads completely
+/// independent from each other so that no thread depends on values
+/// calculated by another thread. The benefit of this approach is that
+/// the algorithm will scale well up to a very large number of CPU
+/// cores. In order to make the threads independent from each other
+/// each thread needs to precompute a lookup table of phi(x, a) values
+/// (this is done in S2_hard_thread(x, y)) every time the thread starts
+/// a new computation.
 ///
 template <typename T, typename FactorTable, typename Primes>
 T S2_hard_OpenMP(T x,
@@ -179,7 +207,7 @@ T S2_hard_OpenMP(T x,
 {
   threads = ideal_num_threads(threads, z);
 
-  LoadBalancer loadBalancer(x, y, z, s2_hard_approx);
+  LoadBalancer loadBalancer(x, z, s2_hard_approx);
   int64_t max_prime = min(y, z / isqrt(y));
   PiTable pi(max_prime);
 
@@ -195,12 +223,15 @@ T S2_hard_OpenMP(T x,
     while (loadBalancer.get_work(&low, &segments, &segment_size, s2_hard, runtime))
     {
       runtime.start();
-      s2_hard = S2_hard_thread(x, y, z, c, low, segments, segment_size, factor, pi, primes, runtime);
+      // Unsigned integer division is usually slightly
+      // faster than signed integer division
+      using UT = typename make_unsigned<T>::type;
+      s2_hard = S2_hard_thread((UT) x, y, z, c, low, segments, segment_size, factor, pi, primes, runtime);
       runtime.stop();
     }
   }
 
-  T s2_hard = (T) loadBalancer.get_result();
+  T s2_hard = (T) loadBalancer.get_sum();
 
   return s2_hard;
 }
@@ -230,8 +261,7 @@ int64_t S2_hard(int64_t x,
   FactorTable<uint16_t> factor(y, threads);
   int64_t max_prime = min(y, z / isqrt(y));
   auto primes = generate_primes<int32_t>(max_prime);
-
-  int64_t s2_hard = S2_hard_OpenMP((intfast64_t) x, y, z, c, (intfast64_t) s2_hard_approx, primes, factor, threads);
+  int64_t s2_hard = S2_hard_OpenMP(x, y, z, c, s2_hard_approx, primes, factor, threads);
 
   print("S2_hard", s2_hard, time);
   return s2_hard;
@@ -265,16 +295,14 @@ int128_t S2_hard(int128_t x,
     FactorTable<uint16_t> factor(y, threads);
     int64_t max_prime = min(y, z / isqrt(y));
     auto primes = generate_primes<uint32_t>(max_prime);
-
-    s2_hard = S2_hard_OpenMP((intfast128_t) x, y, z, c, (intfast128_t) s2_hard_approx, primes, factor, threads);
+    s2_hard = S2_hard_OpenMP(x, y, z, c, s2_hard_approx, primes, factor, threads);
   }
   else
   {
     FactorTable<uint32_t> factor(y, threads);
     int64_t max_prime = min(y, z / isqrt(y));
     auto primes = generate_primes<int64_t>(max_prime);
-
-    s2_hard = S2_hard_OpenMP((intfast128_t) x, y, z, c, (intfast128_t) s2_hard_approx, primes, factor, threads);
+    s2_hard = S2_hard_OpenMP(x, y, z, c, s2_hard_approx, primes, factor, threads);
   }
 
   print("S2_hard", s2_hard, time);
