@@ -1,22 +1,33 @@
 ///
 /// @file  pi_deleglise_rivat2.cpp
-/// @brief Implementation of the Deleglise-Rivat prime counting
-///        algorithm. Compared to pi_deleglise_rivat1.cpp this
-///        version uses compression (FactorTable & PiTable) to
-///        reduce the memory usage.
+/// @brief Simple demonstration implementation of the Deleglise-Rivat
+///        prime counting algorithm. The Deleglise-Rivat algorithm is
+///        an improvement over the Lagarias-Miller-Odlyzko algorithm,
+///        in the Deleglise-Rivat algorithm the special leaves
+///        S2(x, a) have been split up into trivial special leaves,
+///        easy special leaves and hard special leaves.
+///
+///        Deleglise-Rivat formula:
+///        pi(x) = pi(y) + S1(x, a) + S2(x, a) - 1 - P2(x, a)
+///        S2(x, a) = S2_trivial(x, a) + S2_easy(x, a) + S2_hard(x, a)
+///        with y = alpha * x^(1/3), a = pi(y)
+///
+///        This version is identical to pi_deleglise_rivat1.cpp except
+///        that this version uses compression (FactorTable & PiTable)
+///        to reduce the memory usage. pi_deleglise_rivat2.cpp uses up
+///        to 12 times less memory than pi_deleglise_rivat1.cpp.
 ///
 ///        This implementation is based on the paper:
 ///        Tomás Oliveira e Silva, Computing pi(x): the combinatorial
 ///        method, Revista do DETUA, vol. 4, no. 6, March 2006,
 ///        pp. 759-768.
 ///
-/// Copyright (C) 2018 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2019 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
 ///
 
-#include <primecount.hpp>
 #include <primecount-internal.hpp>
 #include <PiTable.hpp>
 #include <FactorTable.hpp>
@@ -64,8 +75,17 @@ void cross_off(int64_t prime,
   multiple = m;
 }
 
-/// Calculate the contribution of the hard special leaves
-/// using a segmented sieve to reduce memory usage.
+/// Compute the contribution of the hard special leaves using a
+/// segmented sieve in order to reduce memory usage.
+///
+/// Note that in the Deleglise-Rivat paper it is suggested to use a
+/// segment size of y. In practice however this uses too much memory
+/// especially when using multi-threading. Hence we are using a
+/// segment size of sqrt(z) as suggested in Xavier Gourdon's paper.
+/// In primecount's implementation a segment size of sqrt(z) seems
+/// ideal since slightly increasing the segment size decreases
+/// performance because of cache misses and slightly decreasing the
+/// segment size also decreases performance.
 ///
 int64_t S2_hard(int64_t x,
                 int64_t y,
@@ -84,7 +104,7 @@ int64_t S2_hard(int64_t x,
   int64_t segment_size = next_power_of_2(isqrt(limit));
   int64_t pi_sqrty = pi[isqrt(y)];
   int64_t pi_sqrtz = pi[min(isqrt(z), y)];
-  int64_t S2_result = 0;
+  int64_t s2_hard = 0;
   double time = get_time();
 
   vector<char> sieve(segment_size);
@@ -97,12 +117,12 @@ int64_t S2_hard(int64_t x,
   {
     // current segment [low, high[
     int64_t high = min(low + segment_size, limit);
-    int64_t b = c + 1;
+    int64_t b = 1;
 
     fill(sieve.begin(), sieve.end(), 1);
 
     // pre-sieve multiples of first c primes
-    for (int64_t b = 1; b <= c; b++)
+    for (; b <= c; b++)
     {
       int64_t k = next[b];
       for (int64_t prime = primes[b]; k < high; k += prime)
@@ -130,12 +150,13 @@ int64_t S2_hard(int64_t x,
 
       for (int64_t m = max_m; m > min_m; m--)
       {
-        if (prime < factor.lpf(m))
+        // mu(m) != 0 && prime < lpf(m)
+        if (prime < factor.mu_lpf(m))
         {
-          int64_t n = prime * factor.get_number(m);
-          int64_t count = tree.count(low, x / n);
-          int64_t phi_xn = phi[b] + count;
-          S2_result -= factor.mu(m) * phi_xn;
+          int64_t pm = prime * factor.get_number(m);
+          int64_t count = tree.count(low, x / pm);
+          int64_t phi_xpm = phi[b] + count;
+          s2_hard -= factor.mu(m) * phi_xpm;
         }
       }
 
@@ -149,7 +170,7 @@ int64_t S2_hard(int64_t x,
     for (; b <= pi_sqrtz; b++)
     {
       int64_t prime = primes[b];
-      int64_t l = pi[min(x / (prime * low), z / prime, y)];
+      int64_t l = pi[min3(x / (prime * low), z / prime, y)];
       int64_t min_hard = max(x / (prime * high), prime);
 
       if (prime >= primes[l])
@@ -157,11 +178,11 @@ int64_t S2_hard(int64_t x,
 
       for (; primes[l] > min_hard; l--)
       {
-        int64_t n = prime * primes[l];
-        int64_t xn = x / n;
-        int64_t count = tree.count(low, xn);
-        int64_t phi_xn = phi[b] + count;
-        S2_result += phi_xn;
+        int64_t pq = prime * primes[l];
+        int64_t xpq = x / pq;
+        int64_t count = tree.count(low, xpq);
+        int64_t phi_xpq = phi[b] + count;
+        s2_hard += phi_xpq;
       }
 
       phi[b] += tree.count(low, high - 1);
@@ -171,8 +192,8 @@ int64_t S2_hard(int64_t x,
     next_segment:;
   }
 
-  print("S2_hard", S2_result, time);
-  return S2_result;
+  print("S2_hard", s2_hard, time);
+  return s2_hard;
 }
 
 /// Calculate the contribution of the special leaves
@@ -181,7 +202,7 @@ int64_t S2(int64_t x,
            int64_t z,
            int64_t c)
 {
-  int64_t s2_trivial = S2_trivial(x, y, z, c, 1);
+  int64_t s2_trivial = S2_trivial(x, y, z, c);
   int64_t s2_easy = S2_easy(x, y, z, c, 1);
   int64_t s2_hard = S2_hard(x, y, z, c);
   int64_t s2 = s2_trivial + s2_easy + s2_hard;
@@ -215,7 +236,7 @@ int64_t pi_deleglise_rivat2(int64_t x)
   print(x, y, z, c, alpha, 1);
 
   int64_t p2 = P2(x, y, 1);
-  int64_t pi_y = pi_legendre(y);
+  int64_t pi_y = pi_legendre(y, 1);
   int64_t s1 = S1(x, y, c, 1);
   int64_t s2 = S2(x, y, z, c);
   int64_t phi = s1 + s2;
