@@ -7,7 +7,17 @@
 ///        Mathematics of Computation, 44 (1985), by J. C. Lagarias,
 ///        V. S. Miller and A. M. Odlyzko.
 ///
-/// Copyright (C) 2018 Kim Walisch, <kim.walisch@gmail.com>
+///        Unlike pi_lmo4.cpp this version does not use a special tree
+///        data structure (a.k.a. Fenwick tree) for counting the number
+///        of unsieved elements but instead counts the number of
+///        unsieved elements directly from the sieve array using the
+///        POPCNT instruction which is much faster.
+///
+///        Lagarias-Miller-Odlyzko formula:
+///        pi(x) = pi(y) + S1(x, a) + S2(x, a) - 1 - P2(x, a)
+///        with y = x^(1/3), a = pi(y)
+///
+/// Copyright (C) 2019 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -51,7 +61,7 @@ int64_t S2(int64_t x,
   vector<int32_t> pi = generate_pi(y);
   vector<int64_t> phi(primes.size(), 0);
 
-  int64_t S2_result = 0;
+  int64_t s2 = 0;
   int64_t pi_sqrty = pi[isqrt(y)];
   int64_t pi_y = pi[y];
 
@@ -62,15 +72,14 @@ int64_t S2(int64_t x,
     int64_t high = min(low + segment_size, limit);
     int64_t low1 = max(low, 1);
 
-    // pre-sieve multiples of first c primes
-    sieve.pre_sieve(c, low, high);
-
+    sieve.pre_sieve(primes, c, low, high);
     int64_t count_low_high = sieve.count((high - 1) - low);
     int64_t b = c + 1;
 
     // For c + 1 <= b <= pi_sqrty
-    // Find all special leaves: n = primes[b] * m
-    // which satisfy:  mu[m] != 0 && primes[b] < lpf[m], low <= (x / n) < high
+    // Find all special leaves in the current segment that are
+    // composed of a prime and a square free number:
+    // low <= x / (primes[b] * m) < high
     for (; b <= pi_sqrty; b++)
     {
       int64_t prime = primes[b];
@@ -86,22 +95,23 @@ int64_t S2(int64_t x,
       {
         if (mu[m] != 0 && prime < lpf[m])
         {
-          int64_t xn = x / (prime * m);
-          int64_t stop = xn - low;
+          int64_t xpm = x / (prime * m);
+          int64_t stop = xpm - low;
           count += sieve.count(i, stop);
           i = stop + 1;
-          int64_t phi_xn = phi[b] + count;
-          S2_result -= mu[m] * phi_xn;
+          int64_t phi_xpm = phi[b] + count;
+          s2 -= mu[m] * phi_xpm;
         }
       }
 
       phi[b] += count_low_high;
-      count_low_high -= sieve.cross_off(b, prime);
+      count_low_high -= sieve.cross_off_count(prime, b);
     }
 
     // For pi_sqrty < b < pi_y
-    // Find all special leaves: n = primes[b] * prime2
-    // which satisfy: low <= (x / n) < high
+    // Find all special leaves in the current segment
+    // that are composed of 2 primes:
+    // low <= x / (primes[b] * primes[l]) < high
     for (; b < pi_y; b++)
     {
       int64_t prime = primes[b];
@@ -115,23 +125,23 @@ int64_t S2(int64_t x,
 
       for (; primes[l] > min_m; l--)
       {
-        int64_t xn = x / (prime * primes[l]);
-        int64_t stop = xn - low;
+        int64_t xpq = x / (prime * primes[l]);
+        int64_t stop = xpq - low;
         count += sieve.count(i, stop);
         i = stop + 1;
-        int64_t phi_xn = phi[b] + count;
-        S2_result += phi_xn;
+        int64_t phi_xpq = phi[b] + count;
+        s2 += phi_xpq;
       }
 
       phi[b] += count_low_high;
-      count_low_high -= sieve.cross_off(b, prime);
+      count_low_high -= sieve.cross_off_count(prime, b);
     }
 
     next_segment:;
   }
 
-  print("S2", S2_result, time);
-  return S2_result;
+  print("S2", s2, time);
+  return s2;
 }
 
 } // namespace
