@@ -2,11 +2,23 @@
 /// @file  pi_lmo_parallel.cpp
 /// @brief Parallel implementation of the Lagarias-Miller-Odlyzko
 ///        prime counting algorithm using OpenMP. This implementation
-///        uses load balancing and counts the number of unsieved
-///        elements using POPCNT without using any special counting
-///        tree data structure.
+///        uses load balancing to ensure all threads are kept busy
+///        till the very end. This implementation also does not use a
+///        special tree data structure (a.k.a. Fenwick tree) for
+///        counting the number of unsieved elements but instead counts
+///        the number of unsieved elements directly from the sieve
+///        array using the POPCNT instruction which is much faster.
 ///
-/// Copyright (C) 2018 Kim Walisch, <kim.walisch@gmail.com>
+///        Lagarias-Miller-Odlyzko formula:
+///        pi(x) = pi(y) + S1(x, a) + S2(x, a) - 1 - P2(x, a)
+///        with y = x^(1/3), a = pi(y)
+///
+///        This implementation is based on the paper:
+///        Tomás Oliveira e Silva, Computing pi(x): the combinatorial
+///        method, Revista do DETUA, vol. 4, no. 6, March 2006,
+///        pp. 759-768.
+///
+/// Copyright (C) 2019 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -53,7 +65,7 @@ int64_t S2_thread(int64_t x,
   int64_t size = pi[min(isqrt(x / low1), y)] + 1;
   int64_t pi_sqrty = pi[isqrt(y)];
   int64_t pi_y = pi[y];
-  int64_t S2_thread = 0;
+  int64_t s2_thread = 0;
 
   if (c >= size - 1)
     return 0;
@@ -70,15 +82,14 @@ int64_t S2_thread(int64_t x,
     int64_t high = min(low + segment_size, limit);
     low1 = max(low, 1);
 
-    // pre-sieve multiples of first c primes
-    sieve.pre_sieve(c, low, high);
-
+    sieve.pre_sieve(primes, c, low, high);
     int64_t count_low_high = sieve.count((high - 1) - low);
     int64_t b = c + 1;
 
     // For c + 1 <= b <= pi_sqrty
-    // Find all special leaves: n = primes[b] * m
-    // which satisfy:  mu[m] != 0 && primes[b] < lpf[m], low <= (x / n) < high
+    // Find all special leaves in the current segment that are
+    // composed of a prime and a square free number:
+    // low <= x / (primes[b] * m) < high
     for (; b <= min(pi_sqrty, size - 1); b++)
     {
       int64_t prime = primes[b];
@@ -94,22 +105,23 @@ int64_t S2_thread(int64_t x,
       {
         if (mu[m] != 0 && prime < lpf[m])
         {
-          int64_t xn = x / (prime * m);
-          int64_t stop = xn - low;
+          int64_t xpm = x / (prime * m);
+          int64_t stop = xpm - low;
           count += sieve.count(i, stop);
           i = stop + 1;
-          int64_t phi_xn = phi[b] + count;
-          S2_thread -= mu[m] * phi_xn;
+          int64_t phi_xpm = phi[b] + count;
+          s2_thread -= mu[m] * phi_xpm;
         }
       }
 
       phi[b] += count_low_high;
-      count_low_high -= sieve.cross_off(b, prime);
+      count_low_high -= sieve.cross_off_count(prime, b);
     }
 
     // For pi_sqrty < b < pi_y
-    // Find all special leaves: n = primes[b] * prime2
-    // which satisfy: low <= (x / n) < high
+    // Find all special leaves in the current segment
+    // that are composed of 2 primes:
+    // low <= x / (primes[b] * primes[l]) < high
     for (; b < min(pi_y, size); b++)
     {
       int64_t prime = primes[b];
@@ -123,30 +135,41 @@ int64_t S2_thread(int64_t x,
 
       for (; primes[l] > min_m; l--)
       {
-        int64_t xn = x / (prime * primes[l]);
-        int64_t stop = xn - low;
+        int64_t xpq = x / (prime * primes[l]);
+        int64_t stop = xpq - low;
         count += sieve.count(i, stop);
         i = stop + 1;
-        int64_t phi_xn = phi[b] + count;
-        S2_thread += phi_xn;
+        int64_t phi_xpq = phi[b] + count;
+        s2_thread += phi_xpq;
       }
 
       phi[b] += count_low_high;
-      count_low_high -= sieve.cross_off(b, prime);
+      count_low_high -= sieve.cross_off_count(prime, b);
     }
 
     next_segment:;
   }
 
-  return S2_thread;
+  return s2_thread;
 }
 
-/// Calculate the contribution of the special leaves.
-/// This is a parallel implementation with advanced load balancing.
-/// As most special leaves tend to be in the first segments we
-/// start off with a small segment size and few segments
-/// per thread, after each iteration we dynamically increase
-/// the segment size and the number of segments.
+/// Calculate the contribution of thes pecial leaves.
+///
+/// This is a parallel S2(x, y) implementation with advanced load
+/// balancing. As most special leaves tend to be in the first segments
+/// we start off with a tiny segment size and one segment per thread.
+/// After each iteration we dynamically increase the segment size (until
+/// it reaches some limit) or the number of segments.
+///
+/// S2(x, y) has been parallelized using an idea devised by Xavier
+/// Gourdon. The idea is to make the individual threads completely
+/// independent from each other so that no thread depends on values
+/// calculated by another thread. The benefit of this approach is that
+/// the algorithm will scale well up to a very large number of CPU
+/// cores. In order to make the threads independent from each other
+/// each thread needs to precompute a lookup table of phi(x, a) values
+/// (this is done in S2_thread(x, y)) every time the thread starts a
+/// new computation.
 ///
 int64_t S2(int64_t x,
            int64_t y,
@@ -164,7 +187,7 @@ int64_t S2(int64_t x,
 
   double time = get_time();
   threads = ideal_num_threads(threads, z);
-  LoadBalancer loadBalancer(x, y, z, s2_approx);
+  LoadBalancer loadBalancer(x, z, s2_approx);
   PiTable pi(y);
 
   #pragma omp parallel for num_threads(threads)
@@ -173,21 +196,21 @@ int64_t S2(int64_t x,
     int64_t low = 0;
     int64_t segments = 0;
     int64_t segment_size = 0;
-    int64_t S2 = 0;
+    int64_t s2 = 0;
     Runtime runtime;
 
-    while (loadBalancer.get_work(&low, &segments, &segment_size, S2, runtime))
+    while (loadBalancer.get_work(&low, &segments, &segment_size, s2, runtime))
     {
       runtime.start();
-      S2 = S2_thread(x, y, z, c, low, segments, segment_size, pi, primes, lpf, mu, runtime);
+      s2 = S2_thread(x, y, z, c, low, segments, segment_size, pi, primes, lpf, mu, runtime);
       runtime.stop();
     }
   }
 
-  int64_t S2 = (int64_t) loadBalancer.get_result();
-  print("S2", S2, time);
+  int64_t s2 = (int64_t) loadBalancer.get_sum();
+  print("S2", s2, time);
 
-  return S2;
+  return s2;
 }
 
 } // namespace
