@@ -1,10 +1,16 @@
 ///
 /// @file  S2_easy_mpi.cpp
 /// @brief Calculate the contribution of the clustered easy leaves
-///        and the sparse easy leaves in parallel using OpenMP
-///        (Deleglise-Rivat algorithm).
+///        and the sparse easy leaves (Deleglise-Rivat algorithm).
+///        This is a distributed implementation using MPI (Message
+///        Passing Interface) and OpenMP multi-threading.
 ///
-/// Copyright (C) 2018 Kim Walisch, <kim.walisch@gmail.com>
+///        This implementation is based on the paper:
+///        Tomás Oliveira e Silva, Computing pi(x): the combinatorial
+///        method, Revista do DETUA, vol. 4, no. 6, March 2006,
+///        pp. 759-768.
+///
+/// Copyright (C) 2019 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -58,9 +64,9 @@ T S2_easy_mpi_master(T x,
   for (int64_t b = max(c, pi_sqrty) + 1 + proc_id; b <= pi_x13; b += procs)
   {
     int64_t prime = primes[b];
-    T x2 = x / prime;
-    int64_t min_trivial = min(x2 / prime, y);
-    int64_t min_clustered = (int64_t) isqrt(x2);
+    T xp = x / prime;
+    int64_t min_trivial = min(xp / prime, y);
+    int64_t min_clustered = (int64_t) isqrt(xp);
     int64_t min_sparse = z / prime;
 
     min_clustered = in_between(prime, min_clustered, y);
@@ -70,27 +76,30 @@ T S2_easy_mpi_master(T x,
     int64_t pi_min_clustered = pi[min_clustered];
     int64_t pi_min_sparse = pi[min_sparse];
 
-    // Find all clustered easy leaves:
-    // n = primes[b] * primes[l]
-    // x / n <= y && phi(x / n, b - 1) == phi(x / m, b - 1)
-    // where phi(x / n, b - 1) = pi(x / n) - b + 2
+    // Find all clustered easy leaves where
+    // successive leaves are identical.
+    // pq = primes[b] * primes[l]
+    // Which satisfy: pq > z && x / pq <= y
+    // where phi(x / pq, b - 1) = pi(x / pq) - b + 2
     while (l > pi_min_clustered)
     {
-      int64_t xn = (int64_t) fast_div(x2, primes[l]);
-      int64_t phi_xn = pi[xn] - b + 2;
-      int64_t xm = (int64_t) fast_div(x2, primes[b + phi_xn - 1]);
-      int64_t l2 = pi[xm];
-      s2_easy += phi_xn * (l - l2);
+      int64_t xpq = fast_div64(xp, primes[l]);
+      int64_t phi_xpq = pi[xpq] - b + 2;
+      int64_t xpq2 = fast_div64(xp, primes[b + phi_xpq - 1]);
+      int64_t l2 = pi[xpq2];
+      s2_easy += phi_xpq * (l - l2);
       l = l2;
     }
 
-    // Find all sparse easy leaves:
-    // n = primes[b] * primes[l]
-    // x / n <= y && phi(x / n, b - 1) = pi(x / n) - b + 2
+    // Find all sparse easy leaves where
+    // successive leaves are different.
+    // pq = primes[b] * primes[l]
+    // Which satisfy: pq > z && x / pq <= y
+    // where phi(x / pq, b - 1) = pi(x / pq) - b + 2
     for (; l > pi_min_sparse; l--)
     {
-      int64_t xn = (int64_t) fast_div(x2, primes[l]);
-      s2_easy += pi[xn] - b + 2;
+      int64_t xpq = fast_div64(xp, primes[l]);
+      s2_easy += pi[xpq] - b + 2;
     }
 
     if (is_print())
