@@ -1,9 +1,8 @@
 ///
 /// @file  S2_hard_mpi.cpp
 /// @brief Calculate the contribution of the hard special leaves using
-///        a prime sieve. This is a distributed implementation
-///        using MPI (Message Passing Interface) and OpenMP
-///        multi-threading.
+///        a prime sieve. This is a distributed implementation using
+///        MPI (Message Passing Interface) and OpenMP multi-threading.
 ///
 ///        Usually the computation of the hard special leaves
 ///        requires a binary indexed tree a.k.a. Fenwick tree to count
@@ -13,7 +12,12 @@
 ///        POPCNT instruction. Hence this implementation does not use
 ///        a binary indexed tree.
 ///
-/// Copyright (C) 2018 Kim Walisch, <kim.walisch@gmail.com>
+///        This implementation is based on the paper:
+///        Tomás Oliveira e Silva, Computing pi(x): the combinatorial
+///        method, Revista do DETUA, vol. 4, no. 6, March 2006,
+///        pp. 759-768.
+///
+/// Copyright (C) 2019 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -44,9 +48,18 @@ using namespace primecount;
 
 namespace {
 
-/// Compute the contribution of the hard special leaves
-/// using a sieve. Each thread processes the interval
-/// [low, low + segments * segment_size[
+/// Compute the contribution of the hard special leaves using a
+/// segmented sieve. Each thread processes the interval
+/// [low, low + segments * segment_size[.
+///
+/// Note that in the Deleglise-Rivat paper it is suggested to use a
+/// segment size of y. In practice however this uses too much memory
+/// especially when using multi-threading. Hence we are using a
+/// segment size of sqrt(z) as suggested in Xavier Gourdon's paper.
+/// In primecount's implementation a segment size of sqrt(z) seems
+/// ideal since slightly increasing the segment size decreases
+/// performance because of cache misses and slightly decreasing the
+/// segment size also decreases performance.
 ///
 template <typename T, typename FactorTable, typename Primes>
 T S2_hard_thread(T x,
@@ -61,14 +74,16 @@ T S2_hard_thread(T x,
                  Primes& primes,
                  Runtime& runtime)
 {
+  T s2_hard = 0;
+  int64_t pi_sqrty = pi[isqrt(y)];
   int64_t low1 = max(low, 1);
   int64_t limit = min(low + segments * segment_size, z + 1);
-  int64_t max_b = pi[min(isqrt(x / low1), isqrt(z), y)];
-  int64_t pi_sqrty = pi[isqrt(y)];
-  T s2_hard = 0;
+  int64_t max_b = pi[min3(isqrt(x / low1), isqrt(z), y)];
+  int64_t min_b = pi[min(z / limit, primes[max_b])];
+  min_b = max(c, min_b) + 1;
 
-  if (c > max_b)
-    return s2_hard;
+  if (min_b > max_b)
+    return 0;
 
   runtime.init_start();
   Sieve sieve(low, segment_size, max_b);
@@ -82,11 +97,9 @@ T S2_hard_thread(T x,
     int64_t high = min(low + segment_size, limit);
     low1 = max(low, 1);
 
-    // pre-sieve multiples of first c primes
-    sieve.pre_sieve(c, low, high);
-
+    sieve.pre_sieve(primes, min_b - 1, low, high);
     int64_t count_low_high = sieve.count((high - 1) - low);
-    int64_t b = c + 1;
+    int64_t b = min_b;
 
     // For c + 1 <= b <= pi_sqrty
     // Find all special leaves: n = primes[b] * m
@@ -94,10 +107,10 @@ T S2_hard_thread(T x,
     for (int64_t end = min(pi_sqrty, max_b); b <= end; b++)
     {
       int64_t prime = primes[b];
-      T x2 = x / prime;
-      int64_t x2_div_high = min(fast_div(x2, high), y);
-      int64_t min_m = max(x2_div_high, y / prime);
-      int64_t max_m = min(fast_div(x2, low1), y);
+      T xp = x / prime;
+      int64_t xp_div_high = min(fast_div(xp, high), y);
+      int64_t min_m = max(xp_div_high, y / prime);
+      int64_t max_m = min(fast_div(xp, low1), y);
       int64_t count = 0;
       int64_t start = 0;
 
@@ -109,21 +122,22 @@ T S2_hard_thread(T x,
 
       for (int64_t m = max_m; m > min_m; m--)
       {
-        if (prime < factor.lpf(m))
+        // mu(m) != 0 && prime < lpf(m)
+        if (prime < factor.mu_lpf(m))
         {
           int64_t fm = factor.get_number(m);
-          int64_t xn = (int64_t) fast_div(x2, fm);
-          int64_t stop = xn - low;
+          int64_t xpm = fast_div64(xp, fm);
+          int64_t stop = xpm - low;
           count += sieve.count(start, stop, low, high, count, count_low_high);
           start = stop + 1;
-          int64_t phi_xn = phi[b] + count;
+          int64_t phi_xpm = phi[b] + count;
           int64_t mu_m = factor.mu(m);
-          s2_hard -= mu_m * phi_xn;
+          s2_hard -= mu_m * phi_xpm;
         }
       }
 
       phi[b] += count_low_high;
-      count_low_high -= sieve.cross_off(b, prime);
+      count_low_high -= sieve.cross_off_count(prime, b);
     }
 
     // For pi_sqrty < b <= pi_sqrtz
@@ -132,11 +146,11 @@ T S2_hard_thread(T x,
     for (; b <= max_b; b++)
     {
       int64_t prime = primes[b];
-      T x2 = x / prime;
-      int64_t x2_div_low = min(fast_div(x2, low1), y);
-      int64_t x2_div_high = min(fast_div(x2, high), y);
-      int64_t l = pi[min(x2_div_low, z / prime)];
-      int64_t min_hard = max(x2_div_high, prime);
+      T xp = x / prime;
+      int64_t xp_div_low = min(fast_div(xp, low1), y);
+      int64_t xp_div_high = min(fast_div(xp, high), y);
+      int64_t l = pi[min(xp_div_low, z / prime)];
+      int64_t min_hard = max(xp_div_high, prime);
       int64_t count = 0;
       int64_t start = 0;
 
@@ -145,16 +159,16 @@ T S2_hard_thread(T x,
 
       for (; primes[l] > min_hard; l--)
       {
-        int64_t xn = (int64_t) fast_div(x2, primes[l]);
-        int64_t stop = xn - low;
+        int64_t xpq = fast_div64(xp, primes[l]);
+        int64_t stop = xpq - low;
         count += sieve.count(start, stop, low, high, count, count_low_high);
         start = stop + 1;
-        int64_t phi_xn = phi[b] + count;
-        s2_hard += phi_xn;
+        int64_t phi_xpq = phi[b] + count;
+        s2_hard += phi_xpq;
       }
 
       phi[b] += count_low_high;
-      count_low_high -= sieve.cross_off(b, prime);
+      count_low_high -= sieve.cross_off_count(prime, b);
     }
 
     next_segment:;
@@ -213,7 +227,10 @@ void S2_hard_slave(T x,
         break;
 
       runtime.start();
-      s2_hard = S2_hard_thread(x, y, z, c, low, segments, segment_size, factor, pi, primes, runtime);
+      // Unsigned integer division is usually slightly
+      // faster than signed integer division
+      using UT = typename make_unsigned<T>::type;
+      s2_hard = S2_hard_thread((UT) x, y, z, c, low, segments, segment_size, factor, pi, primes, runtime);
       runtime.stop();
     }
   }
@@ -227,7 +244,6 @@ void S2_hard_slave(T x,
 ///
 template <typename T>
 T S2_hard_mpi_master(T x,
-                     int64_t y,
                      int64_t z,
                      T s2_hard_approx)
 {
@@ -235,7 +251,7 @@ T S2_hard_mpi_master(T x,
   int slaves = mpi_num_procs() - 1;
 
   MpiMsg msg;
-  MpiLoadBalancer loadBalancer(x, y, z, s2_hard_approx);
+  MpiLoadBalancer loadBalancer(x, z, s2_hard_approx);
   S2Status status(x);
 
   while (slaves > 0)
@@ -283,14 +299,13 @@ int64_t S2_hard_mpi(int64_t x,
   double time = get_time();
 
   if (is_mpi_master_proc())
-    s2_hard = S2_hard_mpi_master(x, y, z, s2_hard_approx);
+    s2_hard = S2_hard_mpi_master(x, z, s2_hard_approx);
   else
   {
     FactorTable<uint16_t> factor(y, threads);
     int64_t max_prime = min(y, z / isqrt(y));
     auto primes = generate_primes<int32_t>(max_prime);
-
-    S2_hard_slave((intfast64_t) x, y, z, c, primes, factor, threads);
+    S2_hard_slave(x, y, z, c, primes, factor, threads);
   }
 
   print("S2_hard", s2_hard, time);
@@ -315,7 +330,7 @@ int128_t S2_hard_mpi(int128_t x,
   double time = get_time();
 
   if (is_mpi_master_proc())
-    s2_hard = S2_hard_mpi_master(x, y, z, s2_hard_approx);
+    s2_hard = S2_hard_mpi_master(x, z, s2_hard_approx);
   else
   {
     // uses less memory
@@ -324,16 +339,14 @@ int128_t S2_hard_mpi(int128_t x,
       FactorTable<uint16_t> factor(y, threads);
       int64_t max_prime = min(y, z / isqrt(y));
       auto primes = generate_primes<uint32_t>(max_prime);
-
-      S2_hard_slave((intfast128_t) x, y, z, c, primes, factor, threads);
+      S2_hard_slave(x, y, z, c, primes, factor, threads);
     }
     else
     {
       FactorTable<uint32_t> factor(y, threads);
       int64_t max_prime = min(y, z / isqrt(y));
       auto primes = generate_primes<int64_t>(max_prime);
-
-      S2_hard_slave((intfast128_t) x, y, z, c, primes, factor, threads);
+      S2_hard_slave(x, y, z, c, primes, factor, threads);
     }
   }
 
