@@ -1,13 +1,20 @@
 ///
 /// @file  pi_lehmer.cpp
+/// @brief Count the number of primes <= x using Lehmer's formula.
+///        Lehmer's formula is an improvement over Meissel's formula,
+///        it adds the P3(x, a) term which is the 3rd partial sieve
+///        function and sets y=x^(1/4) instead of x^(1/3).
 ///
-/// Copyright (C) 2018 Kim Walisch, <kim.walisch@gmail.com>
+///        Lehmer's formula:
+///        pi(x) = pi(y) + phi(x, a) - 1 - P2(x, a) - P3(x, a)
+///        with y = x^1/4, a = pi(y)
+///
+/// Copyright (C) 2019 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
 ///
 
-#include <primecount.hpp>
 #include <primecount-internal.hpp>
 #include <imath.hpp>
 #include <print.hpp>
@@ -26,7 +33,7 @@ int64_t pi_lehmer(int64_t x, int threads)
     return 0;
 
   int64_t y = iroot<4>(x);
-  int64_t a = pi_legendre(y);
+  int64_t a = pi_legendre(y, threads);
 
   print("");
   print("=== pi_lehmer(x) ===");
