@@ -1,13 +1,20 @@
 ///
 /// @file  pi_meissel.cpp
+/// @brief Count the number of primes <= x using Meissel's formula.
+///        Meissel's formula is an improvement over Legendre's formula,
+///        it adds the P2(x, a) term which is the 2nd partial sieve
+///        function and sets y=x^(1/3) instead of x^(1/2).
 ///
-/// Copyright (C) 2018 Kim Walisch, <kim.walisch@gmail.com>
+///        Meissel's formula:
+///        pi(x) = pi(y) + phi(x, a) - 1 - P2(x, a)
+///        with y = x^1/3, a = pi(y)
+///
+/// Copyright (C) 2019 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
 ///
 
-#include <primecount.hpp>
 #include <primecount-internal.hpp>
 #include <imath.hpp>
 #include <print.hpp>
@@ -26,7 +33,7 @@ int64_t pi_meissel(int64_t x, int threads)
     return 0;
 
   int64_t y = iroot<3>(x);
-  int64_t a = pi_legendre(y);
+  int64_t a = pi_legendre(y, threads);
 
   print("");
   print("=== pi_meissel(x) ===");
